import * as j from "jscodeshift";
import { I18nMessages } from "../../../building-blocks/stages/template-model/pieces/getI18nMessages";
/**
 * Find locale file by localeName in file system of generated app
 * then find variable with localeMessages by localeName
 * then merge (add if message with such key not exist) default i18n messages
 * with existing messages to locale file in file system
 *
 * @param localeMessagesPath - path of i18n messages dir
 * @param localeName - using to get locale file in i18n messages dir and find variable with locale messages inside
 * @param defaultI18nMessages - set of default messages of entity, or other item
 * this messages has low priority in merge, and will be added in result set only if there is no
 * messages with such keys in locale file
 */
export declare function mergeI18nMessagesWithLocaleFile(localeMessagesPath: string, localeName: string, defaultI18nMessages: I18nMessages): Promise<void>;
/**
 * Take localeFileContent, parse it to jscodeshift object
 * then find variable with localeMessages by localeName
 * then merge (add if message with such key not exist)  default i18n messages
 * with existing messages to locale file in file system
 *
 * @param localeFileContent - string version of locale file from generated app
 * @param localeName - using to find variable with locale messages inside
 * @param defaultI18nMessages - set of default messages of entity, or other item
 * this messages has low priority in merge, and will be added in result set only if there is no
 * messages with such keys in localeFileContent
 * @returns localeFileContent with added default messages which was missed before
 */
export declare function mergeI18nMessagesWithLocaleFileContent(localeFileContent: string, localeName: string, defaultI18nMessages: I18nMessages): string;
/**
 * Helper util for mergeI18nMessagesWithLocaleFileContent function
 * Existed messages in file with locale messages should be parsed to node for subsequent merge with new messages.
 * @param messages
 */
export declare function parseMessagesToNode(messages: Record<string, any>): j.ObjectExpression;
