"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseMessagesToNode = exports.mergeI18nMessagesWithLocaleFileContent = exports.mergeI18nMessagesWithLocaleFile = void 0;
const deepmerge_1 = __importDefault(require("deepmerge"));
const fs = __importStar(require("fs/promises"));
const j = __importStar(require("jscodeshift"));
const path = __importStar(require("path"));
const getI18nMessages_1 = require("../../../building-blocks/stages/template-model/pieces/getI18nMessages");
const parseTs_1 = require("../../../building-blocks/util/parseTs");
/**
 * Find locale file by localeName in file system of generated app
 * then find variable with localeMessages by localeName
 * then merge (add if message with such key not exist) default i18n messages
 * with existing messages to locale file in file system
 *
 * @param localeMessagesPath - path of i18n messages dir
 * @param localeName - using to get locale file in i18n messages dir and find variable with locale messages inside
 * @param defaultI18nMessages - set of default messages of entity, or other item
 * this messages has low priority in merge, and will be added in result set only if there is no
 * messages with such keys in locale file
 */
async function mergeI18nMessagesWithLocaleFile(localeMessagesPath, localeName, defaultI18nMessages) {
    const localePath = path.join(localeMessagesPath, `${localeName}.ts`);
    const localeFile = await fs.readFile(localePath, "utf-8")
        .catch(() => {
        throw new Error(`File with '${localeName}' locale not found in '${localeMessagesPath}' dir`);
    });
    const localeFileContent = mergeI18nMessagesWithLocaleFileContent(localeFile, localeName, defaultI18nMessages);
    await fs.writeFile(localePath, localeFileContent, "utf-8");
}
exports.mergeI18nMessagesWithLocaleFile = mergeI18nMessagesWithLocaleFile;
/**
 * Take localeFileContent, parse it to jscodeshift object
 * then find variable with localeMessages by localeName
 * then merge (add if message with such key not exist)  default i18n messages
 * with existing messages to locale file in file system
 *
 * @param localeFileContent - string version of locale file from generated app
 * @param localeName - using to find variable with locale messages inside
 * @param defaultI18nMessages - set of default messages of entity, or other item
 * this messages has low priority in merge, and will be added in result set only if there is no
 * messages with such keys in localeFileContent
 * @returns localeFileContent with added default messages which was missed before
 */
function mergeI18nMessagesWithLocaleFileContent(localeFileContent, localeName, defaultI18nMessages) {
    const source = parseTs_1.parseTs(localeFileContent);
    const localeVariable = source.findVariableDeclarators(localeName);
    if (localeVariable.length === 0) {
        throw new Error(`${localeName} variable not found in file with ${localeName} locale`);
    }
    localeVariable
        .find(j.ObjectExpression)
        .at(0)
        .replaceWith((path) => {
        return parseMessagesToNode(deepmerge_1.default(defaultI18nMessages, parseNodeToMessages(path.node) || {}));
    });
    return source.toSource();
}
exports.mergeI18nMessagesWithLocaleFileContent = mergeI18nMessagesWithLocaleFileContent;
/**
 * Helper util for mergeI18nMessagesWithLocaleFileContent function
 * Existed messages in file with locale messages should be parsed to node for subsequent merge with new messages.
 * @param messages
 */
function parseMessagesToNode(messages) {
    return j.objectExpression(Object.keys(messages)
        // 'deepmerge' library change order of SpreadElement elements.
        // SpreadElement should be first in js objects after deepmerge
        .sort((key) => (messages[key].__type__ === "SpreadElement" ? -1 : 1))
        .map((key) => {
        if (messages[key].__type__ === "SpreadElement") {
            return j.spreadElement(j.identifier(key));
        }
        return j.objectProperty(j.identifier(key), typeof messages[key] === "string" ? j.literal(messages[key]) : parseMessagesToNode(messages[key]));
    }));
}
exports.parseMessagesToNode = parseMessagesToNode;
/**
 * Helper util for mergeI18nMessagesWithLocaleFileContent function
 * Parse jscodeshift ObjectExpression node to javascript object with i18n messages
 * @param node
 */
// TODO add test
function parseNodeToMessages(node) {
    // node is object property
    if (j.ObjectProperty.check(node)) {
        const messageKey = j.Identifier.check(node.key) ? node.key.name : undefined;
        if (messageKey == null) {
            return {};
        }
        let messageValue;
        // node is Literal
        if (j.Literal.check(node.value)) {
            messageValue = typeof node.value.value === "string" ? node.value.value : undefined;
            // node is object, but not literal - recursive parseNodeToMessages
        }
        else {
            messageValue = parseNodeToMessages(node.value) || undefined;
        }
        return messageValue != null ? { [messageKey]: messageValue } : {};
    }
    // node is object expression - recursive parsing for nested nodes
    if (j.ObjectExpression.check(node)) {
        return node.properties.reduce((acc, cur) => ({ ...acc, ...parseNodeToMessages(cur) }), {});
    }
    // node is spread element
    if (j.SpreadElement.check(node)) {
        // check that spread argument is identifier
        const messageKey = j.Identifier.check(node.argument) ? node.argument.name : undefined;
        return messageKey != null ? { [messageKey]: new getI18nMessages_1.SpreadElement() } : {};
    }
    // not supported type of node - return empty object
    return {};
}
//# sourceMappingURL=mergeI18nMessagesWithLocaleFile.js.map